
import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import javax.xml.datatype.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

public class Applicazione {
    private List<Auto> vetture;
    private boolean parsed;

    public Applicazione() {
        vetture = new ArrayList<>();
        parsed = false;
    }
    
    public List<Auto> parseDocument(String filename) throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilderFactory factory;
        DocumentBuilder builder;
        Document document;
        Element root, element;
        NodeList nodelist;
        Auto auto;
        
        // validazione documento XML
        try {
         Validate.validate(filename, "schema.xsd");
        }
        catch (SAXException exception) {
         return null;
        }
        // creazione dell’albero DOM dal documento XML
        factory = DocumentBuilderFactory.newInstance();
        builder = factory.newDocumentBuilder();
        document = builder.parse(filename);
        root = document.getDocumentElement();
        // generazione della lista degli elementi "auto"
        nodelist = root.getElementsByTagName("auto");
        if (nodelist != null && nodelist.getLength() > 0) {
            for (int i=0; i<nodelist.getLength(); i++) {
                element = (Element)nodelist.item(i);
                auto = getAuto(element);
                vetture.add(auto);
            }
        }
        parsed = true;
        return vetture;
    }
    
    private Auto getAuto(Element element) {
        List<Agenzia> agenzie = new ArrayList<>();
        Auto auto;
        Agenzia agenzia;
        NodeList nodelist;
        Element node;
        
        String classe = getTextValue(element, "classe");
        String marca = getTextValue(element, "marca");
        String modello = getTextValue(element, "modello");
        int cilindrata = getIntValue(element, "cilindrata");
        int potenza = getIntValue(element, "potenza");
        String alimentazione = getTextValue(element, "alimentazione");
        int passeggeri = getIntValue(element, "numero_passeggeri");
        int bagagliaio = getIntValue(element, "volume_bagagliaio");
        // generazione della lista degli elementi "agenzia"
        nodelist = element.getElementsByTagName("agenzie");
        node = (Element)(nodelist.item(0));
        nodelist = node.getElementsByTagName("agenzia");
        if (nodelist != null && nodelist.getLength() > 0) {
            for (int i=0; i<nodelist.getLength(); i++) {
                node = (Element)nodelist.item(i);
                agenzia = getAgenzia(node);
                agenzie.add(agenzia);
            }
        }
        auto = new Auto(classe, marca, modello, cilindrata, potenza, alimentazione, passeggeri, bagagliaio, agenzie);
        return auto;
    }
    
    private Agenzia getAgenzia(Element element) {
        Agenzia agenzia;
        
        String denominazione = element.getAttribute("denominazione");
        String città = element.getAttribute("città");
        agenzia = new Agenzia(denominazione, città);
        return agenzia;
    }
    
    // restituisce il valore testuale dell’elemento figlio specificato
    private String getTextValue(Element element, String tag) {
        String value = null;
        NodeList nodelist;
        nodelist = element.getElementsByTagName(tag);
        if (nodelist != null && nodelist.getLength() > 0) {
            value = nodelist.item(0).getFirstChild().getNodeValue();
        }
        return value;
    }

    // restituisce il valore intero dell’elemento figlio specificato
    private int getIntValue(Element element, String tag) {
        return Integer.parseInt(getTextValue(element, tag));
    }

   
    public Collection<Agenzia> noleggiaIbrida() {
        HashMap<String, Agenzia> risultato = new HashMap<>();
        
        if (!parsed)
          return null;
        
        Iterator iterator = vetture.iterator();
        while (iterator.hasNext()) {
             Auto auto = (Auto)iterator.next();
             if (auto.getAlimentazione().equalsIgnoreCase("Ibrida")) {
                 Agenzia[] agenzie = auto.getAgenzie();
                 for (int n=0; n<agenzie.length; n++)
                     risultato.put(agenzie[n].getDenominazione()+agenzie[n].getCittà(), agenzie[n]);
             }
        }
        return risultato.values();
    }
    
    public static void main(String[] args) {
        Collection<Agenzia> agenzie = null;
        Applicazione app = new Applicazione();
        try {
            app.parseDocument("documento.xml");
            agenzie = app.noleggiaIbrida();
            System.out.println("Numero di agenzie: " + agenzie.size());
            Iterator iterator = agenzie.iterator();
            while (iterator.hasNext()) {
                 System.out.println(iterator.next().toString());
            }
        }
        catch (ParserConfigurationException | SAXException | IOException exception) {
            System.out.println("Errore!");
        }
    }
}
